<?php

namespace Drupal\something_went_wrong;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Config\ImmutableConfig;
use Drupal\Core\Language\LanguageManagerInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\Core\Mail\MailManagerInterface;
use Drupal\Core\StringTranslation\StringTranslationTrait;
use GuzzleHttp\Client;

/**
 * Service Provider for Something went wrong.
 *
 * @package Drupal\something_went_wrong
 */
class SomethingWentWrongService implements SomethingWentWrongServiceInterface {

  use StringTranslationTrait;

  /**
   * The module config.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected ImmutableConfig $moduleConfig;

  /**
   * The site config.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected ImmutableConfig $siteConfig;

  /**
   * The mail manager.
   *
   * @var \Drupal\Core\Mail\MailManagerInterface
   */
  protected MailManagerInterface $managerMail;

  /**
   * The logger.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected LoggerChannelInterface $logger;

  /**
   * The language manager.
   *
   * @var \Drupal\Core\Language\LanguageManagerInterface
   */
  protected LanguageManagerInterface $languageManager;

  /**
   * SomethingWentWrongExceptionSubscriber constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\Core\Mail\MailManagerInterface $manager_mail
   *   The mail manager.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger
   *   The logger.
   * @param \Drupal\Core\Language\LanguageManagerInterface $language_manager
   *   The language manager.
   */
  public function __construct(
    ConfigFactoryInterface $config_factory,
    MailManagerInterface $manager_mail,
    LoggerChannelFactoryInterface $logger,
    LanguageManagerInterface $language_manager,
  ) {
    $this->moduleConfig = $config_factory->get('something_went_wrong.settings');
    $this->siteConfig = $config_factory->get('system.site');
    $this->managerMail = $manager_mail;
    $this->logger = $logger->get('something_went_wrong');
    $this->languageManager = $language_manager;
  }

  /**
   * {@inheritDoc}
   */
  public function buildMessage(\Throwable $exception, bool $html = FALSE): string {
    $class_name = get_class($exception);
    $exception_message = $exception->getMessage();
    $exception_file = $exception->getFile();
    $exception_line = $exception->getLine();

    $new_line = "\n";
    if ($html) {
      $new_line = "<br />";
    }

    return sprintf(
      "Uncaught PHP Exception: $new_line Class name: %s $new_line Message: %s $new_line File: %s $new_line Line: %s",
      $class_name,
      $exception_message,
      $exception_file,
      $exception_line,
    );
  }

  /**
   * {@inheritDoc}
   */
  public function sendMessageToSlack(string $message): void {
    $label = $this->moduleConfig->get('label');
    $webhook_url = $this->moduleConfig->get('webhook_url');

    if ($webhook_url) {
      $client = new Client();

      try {
        $client->post($webhook_url, [
          'body' => json_encode([
            'text' => $message,
            'username' => $label ?: $this->t('Something went wrong Bot'),
          ]),
        ]);
      }
      catch (\Exception $e) {
        $this->logger->error('Error while sending Slack message. Error: ' . $e->getMessage());
      }
    }
  }

  /**
   * {@inheritDoc}
   */
  public function sendMessageToMail(string $message): void {
    $mail_custom_address = $this->moduleConfig->get('mail_custom_address');

    if ($mail_custom_address) {
      $email = $mail_custom_address;
    }
    else {
      $email = $this->siteConfig->get('mail');
    }

    if ($email) {
      $params['message'] = $message;
      $language = $this->languageManager->getCurrentLanguage()->getId();
      $this->managerMail->mail('something_went_wrong', 'sww_error_mail', $email, $language, $params, NULL, TRUE);
    }
  }

}
